# Generated by Django 5.2.1 on 2025-05-14 19:56

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('path', models.CharField(max_length=255, primary_key=True, serialize=False)),
            ],
        ),
    ]
